#!/usr/bin/perl
# $Id: am_disk.pl,v 1.1 2001/08/10 09:50:13 jcheng Exp $
# FIXME: This sends email to admin if it's been over 1 day FOR A USER.
# might it be better to only send email once a day about this,
# say at 5am? Then, the admin would only get email once a day
# no matter how many emails.

use lib qw( /usr/sausalito/perl /usr/sausalito/perl/Base);
use AM::Util;
use strict;
use Disk;
use CCE;
use SendEmail;

my $DEBUG = 0;

if (!$ENV{red_free}) {
    $ENV{red_free} = 100;
}

if (!$ENV{red_pcnt}) {
    $ENV{red_pcnt} = 95;
}

if (!$ENV{yellow_free}) {
    $ENV{yellow_free} = 125;
}

if (!$ENV{yellow_pcnt}) {
    $ENV{yellow_pcnt} = 90;
}

my %am_states = am_get_statecodes();
my @dev_warnings = ();

# get mounts from df | grep "^/dev/"
# check if device is readwrite in /proc/mounts
# if more than $red_percent used or less than $red_free available, then red
# if more than $yellow_percent used or less than $yellow_free available, then yellow
my ($rw, $dev_status, $worst_dev_status, $server_status);
my $worst_dev_status = $am_states{AM_STATE_GREEN};

open (DF, "/bin/df |");
while (<DF>) {
    chomp;
    if (/^\/dev\//) {
	my ($device, $size, $used, $avail, $percent, $mount) = split(/ +/);
	$rw = `grep $device /proc/mounts`;
	if (!$rw) {
	    next;
	}

	$percent =~ s/%//;
	$avail = $avail / 1024; # AM thresholds are in megs, df is in kbytes

	if (($percent > $ENV{red_pcnt}) || ($avail < $ENV{red_free})) {
	    $dev_status = $am_states{AM_STATE_RED};
	    push @dev_warnings, "[[base-disk.amDiskWarning,fs=\"$mount\",pcnt=\"$percent\",free=\"$avail\"]]"; 
	} elsif (($percent > $ENV{yellow_pcnt}) || ($avail < $ENV{yellow_pcnt})) {
	    push @dev_warnings, "[[base-disk.amDiskWarning,fs=\"$mount\",pcnt=\"$percent\",free=\"$avail\"]]"; 
	    $dev_status = $am_states{AM_STATE_YELLOW};
	}

	$worst_dev_status = $worst_dev_status > $dev_status ? $worst_dev_status : $dev_status;
    }
}

$server_status = $worst_dev_status;

my $cce = new CCE;
$cce->connectuds();

my ($am_oid) = $cce->findx('ActiveMonitor');
my ($ok, $am) = $cce->get($am_oid, 'Disk');

my @site_warnings = ();
my @user_warnings = ();
my @users_to_warn = ();

# for all vsites $site
# find all users in $site
# refresh user.disk info
# if user.disk is over quota, then user.disk is red
# if user.disk is red, then site.disk is yellow

# if site is over quota, then site.disk is red
# if site.disk is red, then server is yellow

my ($site_ok, $site, $name, $fqdn, $old_site_status, $site_status, $worst_site_status);

my @site_oids = $cce->findx('Vsite');
foreach my $a_site (@site_oids) {
    # checking site usage
    # look a old usage
    # look at new usage
    # if new is over quota, then state = red;
    # look at users
    # if user over quota, then state = yellow;
    # if new is worse than old, then mail;

    # checking old usage
    # site name is in empty namespace
    ($site_ok, $site) = $cce->get($a_site);
    $name = $site->{name};
    $fqdn = $site->{fqdn};

    $DEBUG && print "checking site $a_site quotas\n";
    ($site_ok, $site) = $cce->get($a_site, 'Disk');
    $old_site_status = is_over_quota($site);

    # checking new usage
    ($site_ok) = $cce->set($a_site, 'Disk', { 'refresh' => time() } );
    ($site_ok, $site) = $cce->get($a_site, 'Disk');
#    $site->{used} = 513025;
    $site_status = is_over_quota($site);

    my @users = $cce->findx('User' , { 'site' => $name});

    my ($user_ok, $user, $old_user_status, $user_status, $worst_user_status);

    foreach my $a_user (@users) {

	$DEBUG && print "checking user $a_user quotas\n";

	# checking old usage
	($user_ok, $user) = $cce->get($a_user, 'Disk');
	$old_user_status = is_over_quota($user);

	# checking new usage
	($user_ok) = $cce->set($a_user, 'Disk', { 'refresh' => time() });
	($user_ok, $user) = $cce->get($a_user, 'Disk');
#	$user->{used} = 20481;
	$user_status = is_over_quota($user);

	# if new is worse than old, then mail
	if ($user_status == $am_states{AM_STATE_RED} && # we're red AND
	    ($user_status > $old_user_status || # either we are worse now
	    $user->{lastmailed} < time - 3600*24)) { # or it's been a day since we mailed them
	    ($user_ok) = $cce->set($a_user, 'Disk', { 'lastmailed' => time });
	    ($user_ok, $user) = $cce->get($a_user);
	    if ($am->{mail_admin_on_user}) {
		push @user_warnings, $user->{name};
	    }
	    if ($am->{mail_user}) {
		push @users_to_warn, $user->{name} . '@' . $fqdn;
	    }
	}

	$DEBUG && print "user $a_user is $user_status|\n";
	# keep track of the worst state for all users in this site
	$worst_user_status = $worst_user_status > $user_status ? $worst_user_status : $user_status;
    }

    # if user is over quota, then site state = yellow
    if ($worst_user_status == $am_states{AM_STATE_RED}) {
	$site_status = $site_status > $am_states{AM_STATE_YELLOW} ? $site_status : $am_states{AM_STATE_YELLOW};
	($site_ok) = $cce->set($a_site, 'Disk', { 'user_over_quota' => 1 });
    } else {
	($site_ok) = $cce->set($a_site, 'Disk', { 'user_over_quota' => 0 });	
    }

    # if new is worse than old, then mail
    if ($site_status == $am_states{AM_STATE_RED} && # we're red
	$am->{mail_admin_on_vsite} && # and we're supposed to mail admin
	($site_status > $old_site_status || # and either we are worse now
	$site->{lastmailed} < time - 3600*24)) { # OR it's been a day since we mailed 
	    ($site_ok) = $cce->set($a_site, 'Disk', { 'lastmailed' => time });
	    push @site_warnings, $fqdn;
	}

    $DEBUG && print "site $a_site is $site_status|\n";
    # keep track of the worst state for all sites
    $worst_site_status = $worst_site_status > $site_status ? $worst_site_status : $site_status;
}

my $site_over_quota = 0;

if ($worst_site_status == $am_states{AM_STATE_RED}) {
    $server_status = $server_status > $am_states{AM_STATE_YELLOW} ? $server_status : $am_states{AM_STATE_YELLOW};
    $site_over_quota = 1;
}

# this gets weird
# AM output will only ever show up in the email ONCE, because
# AM only sends mail on state changes

# but we also want our email to get sent once a day to the people involved
# so that mail output needs to be handled seperately

my @am_output = ();
my @mail_output = ();

# mail warnings are warnings about vsites or users that are over quota
if (@site_warnings) {
    push @mail_output, "[[base-disk.sites_over_quota,sites=\"" . join(',', @site_warnings) . "\"]]";
}
if (@user_warnings) {
    push @mail_output, "[[base-disk.users_over_quota,users=\"" . join(',', @user_warnings) . "\"]]";
}

# mail to admin
if (@mail_output) {
    use Data::Dumper;

    $DEBUG && print "mailing to admin" . join(',', @mail_output);

    my ($ok, $am) = $cce->get($am_oid);
    my @am_recips = $cce->scalar_to_array($am->{alertEmailList});
    my $recips = join(',', @am_recips);
  SendEmail::sendEmail($recips, 'admin', 
		       '[[base-disk.userOverQuota]]', join("\n", @mail_output));
}

# mail to users
foreach my $user (@users_to_warn) {
  SendEmail::sendEmail($user, 'admin', '[[base-disk.userOverQuota]]', 
		       '[[base-disk.overQuotaMsg]]');
}

# AM warnings include server usage
# and a simple note that a site is over quota and to look for a 2nd email about this
if (@dev_warnings) {
    push @am_output, @dev_warnings;
} 
if ($site_over_quota) {
    push @am_output, '[[base-disk.site_over_quota]]';
}

if (!@am_output) {
    push @am_output, '[[base-disk.amDiskOk]]';
}

print join("\n", @am_output);
exit $server_status;

# mailing?
# if user is newly over quota and it's once a day and we're supposed to mail user, then mail user
# if user is newly over quota and it's once a day and we're supposed to mail admin, then mail admin

# if site is newly over quota and it's once a day and we're supposed to mail admin, then mail admin

sub is_over_quota {
    my $obj = shift;
    my ($used, $quota);
    $used = $obj->{used};
    $quota = $obj->{quota};
    $quota *= 1024; # quota is in megs, usage is in kbytes

    # if unlimited quota, then quota = -1
    if ($quota > 0 && $used >= $quota) {
#	print "usage is $used, over quota $quota\n";
	return $am_states{AM_STATE_RED};
    } else {
#	print "usage is $used, under quota $quota\n";
	return $am_states{AM_STATE_GREEN};
    }
}

